@@ -36,7 +36,10 @@
 #include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-common/tracker-utils.h>
+
+#if HAVE_TRACKER_FTS
 #include <libtracker-fts/tracker-fts.h>
+#endif
 
 #include "tracker-db-journal.h"
 #include "tracker-db-manager.h"
@@ -994,10 +997,14 @@ tracker_db_manager_shutdown (void)
 	sys_tmp_dir = NULL;
 	g_free (sql_dir);
 
+#if HAVE_TRACKER_FTS
 	/* shutdown fts in all threads
-	   needs to be done before shutting down all db interfaces as
-	   shutdown does not happen in thread where interface was created */
+	 * needs to be done before shutting down all db interfaces as
+	 * shutdown does not happen in thread where interface was created
+	 */
 	tracker_fts_shutdown_all ();
+#endif
+
 	/* shutdown db interfaces in all threads */
 	g_static_private_free (&interface_data_key);
 
@@ -1352,13 +1359,12 @@ tracker_db_manager_set_first_index_done (gboolean done)
 		}
 	} else if (!done && already_exists) {
 		/* If NOT done, remove stamp file */
+		g_message ("  Removing first index file:'%s'", filename);
+
 		if (g_remove (filename)) {
-			g_warning ("  Could not remove file:'%s', %s", 
+			g_warning ("    Could not remove file:'%s', %s",
 			           filename,
 			           g_strerror (errno));
-		} else {
-			g_message ("  First index file:'%s' removed",
-			           filename);
 		}
 	}
 
@@ -1437,13 +1443,12 @@ tracker_db_manager_set_last_crawl_done (gboolean done)
 		g_free (content);
 	} else if (!done && already_exists) {
 		/* If NOT done, remove stamp file */
+		g_message ("  Removing last crawl file:'%s'", filename);
+
 		if (g_remove (filename)) {
-			g_warning ("  Could not remove file:'%s', %s", 
+			g_warning ("    Could not remove file:'%s', %s",
 			           filename,
 			           g_strerror (errno));
-		} else {
-			g_message ("  Last crawl file:'%s' removed",
-			           filename);
 		}
 	}
 
