@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2007,      Jason Kivlighn (jkivlighn@gmail.com)
- * Copyright (C) 2008-2009, Nokia
+ * Copyright (C) 2007, Jason Kivlighn <jkivlighn@gmail.com>
+ * Copyright (C) 2008-2009, Nokia <ivan.frade@nokia.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -25,22 +25,24 @@
 #include <glib.h>
 
 #include <libxml/HTMLparser.h>
-
-#include <libtracker-common/tracker-ontology.h>
-
+#include <libtracker-common/tracker-utils.h>
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
 
-#define RDF_TYPE TRACKER_RDF_PREFIX "type"
+#include "tracker-main.h"
 
 typedef enum {
 	READ_TITLE,
+	READ_IGNORE
 } tag_type;
 
 typedef struct {
 	TrackerSparqlBuilder *metadata;
 	tag_type current;
 	const gchar *uri;
+	guint in_body : 1;
+	GString *title;
+	GString *plain_text;
+	guint n_bytes_remaining;
 } parser_data;
 
 static void extract_html (const gchar          *filename,
@@ -175,9 +177,27 @@ parser_start_element (void           *data,
 				}
 			}
 		}
+	} else if (g_ascii_strcasecmp (name, "body") == 0) {
+		pd->in_body = TRUE;
+	} else if (g_ascii_strcasecmp (name, "script") == 0) {
+		/* Ignore javascript and such */
+		pd->current = READ_IGNORE;
 	}
 }
 
+static void
+parser_end_element (void           *data,
+                    const xmlChar  *name_)
+{
+	parser_data *pd = data;
+	const gchar *name = (const gchar*) name_;
+
+        if (g_ascii_strcasecmp (name, "title") == 0 ||
+            g_ascii_strcasecmp (name, "script") == 0) {
+                    pd->current = -1;
+        }
+}
+
 static void
 parser_characters (void          *data,
                    const xmlChar *ch,
@@ -187,14 +207,37 @@ parser_characters (void          *data,
 
 	switch (pd->current) {
 	case READ_TITLE:
-		tracker_sparql_builder_predicate (pd->metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (pd->metadata, ch);
+		g_string_append (pd->title, ch);
+		break;
+	case READ_IGNORE:
 		break;
 	default:
+		if (pd->in_body && pd->n_bytes_remaining > 0) {
+			gsize text_len;
+
+			text_len = strlen (ch);
+
+			if (tracker_text_validate_utf8 (ch,
+			                                (pd->n_bytes_remaining < text_len ?
+			                                 pd->n_bytes_remaining :
+			                                 text_len),
+			                                &pd->plain_text,
+			                                NULL)) {
+				/* In the case of HTML, each string arriving this
+				 * callback is independent to any other previous
+				 * string, so need to add an explicit whitespace
+				 * separator */
+				g_string_append_c (pd->plain_text, ' ');
+			}
+
+			if (pd->n_bytes_remaining > text_len) {
+				pd->n_bytes_remaining -= text_len;
+			} else {
+				pd->n_bytes_remaining = 0;
+			}
+		}
 		break;
 	}
-
-	pd->current = -1;
 }
 
 static void
@@ -202,6 +245,7 @@ extract_html (const gchar          *uri,
               TrackerSparqlBuilder *preupdate,
               TrackerSparqlBuilder *metadata)
 {
+	TrackerConfig *config;
 	htmlDocPtr doc;
 	parser_data pd;
 	gchar *filename;
@@ -221,7 +265,7 @@ extract_html (const gchar          *uri,
 		NULL, /* startDocument */
 		NULL, /* endDocument */
 		parser_start_element, /* startElement */
-		NULL, /* endElement */
+		parser_end_element, /* endElement */
 		NULL, /* reference */
 		parser_characters, /* characters */
 		NULL, /* ignorableWhitespace */
@@ -241,11 +285,17 @@ extract_html (const gchar          *uri,
 	};
 
 	tracker_sparql_builder_predicate (metadata, "a");
-	tracker_sparql_builder_object (metadata, "nfo:Document");
+	tracker_sparql_builder_object (metadata, "nfo:HtmlDocument");
 
 	pd.metadata = metadata;
 	pd.current = -1;
+	pd.in_body = FALSE;
 	pd.uri = uri;
+	pd.plain_text = g_string_new (NULL);
+	pd.title = g_string_new (NULL);
+
+	config = tracker_main_get_config ();
+	pd.n_bytes_remaining = tracker_config_get_max_bytes (config);
 
 	filename = g_filename_from_uri (uri, NULL, NULL);
 	doc = htmlSAXParseFile (filename, NULL, &handler, &pd);
@@ -254,6 +304,24 @@ extract_html (const gchar          *uri,
 	if (doc) {
 		xmlFreeDoc (doc);
 	}
+
+	g_strstrip (pd.plain_text->str);
+	g_strstrip (pd.title->str);
+
+	if (pd.title->str &&
+	    *pd.title->str != '\0') {
+		tracker_sparql_builder_predicate (metadata, "nie:title");
+		tracker_sparql_builder_object_unvalidated (metadata, pd.title->str);
+	}
+
+	if (pd.plain_text->str &&
+	    *pd.plain_text->str != '\0') {
+		tracker_sparql_builder_predicate (metadata, "nie:plainTextContent");
+		tracker_sparql_builder_object_unvalidated (metadata, pd.plain_text->str);
+	}
+
+	g_string_free (pd.plain_text, TRUE);
+	g_string_free (pd.title, TRUE);
 }
 
 TrackerExtractData *
